class String
   def underscore
     gsub(/(.)([A-Z])/,'\1_\2').downcase
   end
end

class Nanopop::Picture

  URL         = "#{Nanopop::ASSETS_URL}/tasks/%s/%s"
  AVATAR_URL  = "#{Nanopop::ASSETS_URL}/users/%s/%s"

  def initialize ats
    @ats = ats
  end

  def url
    URL % [adjust(task_id), picture_filename]
  end

  def avatar_url
    AVATAR_URL % [adjust(user_id), user_picture_filename]
  end

  def self.latest qty
    raise Nanopop::UndefinedToken unless Nanopop.configuration.token

    response = fetch(:latest, numberOfPictures: qty)
    doc = Nokogiri.XML(response)

    doc.search('PictureLatest').map do |item|
      ats = {}
      item.element_children.each do |at|
        key = at.name.scan(/_x003C_(.+)_x003E_k__BackingField/).flatten.first.underscore
        value = at.text
        ats[key.to_sym] = value
      end
      new(ats)
    end
  end

  private

  def self.query_string hash
    uri = Addressable::URI.new
    uri.query_values = hash
    uri.query
  end

  def self.fetch method, params={}
    url = "#{Nanopop::API_URL}/picture/#{method}/?#{query_string(params)}"
    RestClient.get(url, {"Authorization-Token" => Nanopop.configuration.token})
  end

  def adjust value
    value.rjust(7, '0')
  end

  def method_missing method, params={}
    @ats[method]
  end

end
